/***********************************************************************
ColorSafe 1.0 Beta - http://www.woven-media.com/
Copyright (C) 2008-2009, Brent Allen Parrish <info@woven-media.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
************************************************************************/
package com.woven_media.colorsafe;

import java.awt.Component;
import java.awt.Toolkit;
import java.awt.HeadlessException;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.IOException;
// Swingx
import javax.swing.JPopupMenu;
import javax.swing.JFrame;
import javax.swing.JTextField;
import javax.swing.JMenuItem;

/**
 * AbstractPopup class provides an abstraction for <code>javax.swing.JPopup</code>
 * menus. A <code>Template Pattern</code> form has been applied so common cut, copy and paste commands
 * can be easily implemented in classes that extend AbstractPopup class.
 * AbstractPopup handles gaining access to the user's system clipboard
 * by following the recommendations in the Java 6 SE documentation
 * (Sun MicroSystems, Inc.) to check for an install of the <code>java.lang.SecurityManager</code>
 * when calling <a href="http://java.sun.com/javase/6/docs/api/java/awt/Toolkit.html#getSystemClipboard()">
 * getSystemClipboard()</a>. If security checks are passed, the static member <code>clipboard</code>
 * is set and is made available to all classes extending AbstractPopup.
 *
 * @author  Brent Allen Parrish
 * @version %I%, %G%
 * @since   1.0
 * @see     <a href="http://en.wikipedia.org/wiki/Template_method_pattern">Template Pattern</a>
 * @see     java.lang.SecurityManager
 * @see     com.woven_media.colorsafe.CutNPastePopup
 * @see     com.woven_media.colorsafe.AbstractPopup
 */
public abstract class AbstractPopup extends JPopupMenu implements ActionListener
{
    // Private members
    private Component host;
    private Component component;
    private JMenuItem[] menuItems;
    // Protected
    protected static Clipboard clipboard;

    /**
     * Class constructor sets the <code>java.awt.Component parent</code> member
     * with a reference to this instance's <code>parent</code> component. A
     * <code>java.awt.Component child</code> reference to the component that the
     * popup will be attached to is set in the <code>component</code>
     * member. Calls the <code>init</code> method.
     *
     * @param host      <code>java.awt.Component</code> reference
     *                  to the <code>parent</code> component.
     * @param component <code>java.awt.Component</code> reference
     *                  to the <code>child</code> component.
     * @since           1.0
     */
    public AbstractPopup(Component host, Component component)
    {
        this.host = host;
        this.component = component;
        init();
    }

    /*
     * Initializes the menu building process and starts the security checks.
     *
     * @since        1.0
     */
    private void init()
    {
        buildMenu();
        getClipboard();
    }

   /**
    * Builds the popup by grabbing the resource strings for the
    * menu commands, setting the action commands and adding the
    * <code>java.awt.event.ActionListener</code> event handler.
    *
    * @since 1.0
    */
   protected void buildMenu()
   {
       // CutNPaste Popup menu
       menuItems = new JMenuItem[3];
       // TODO: Localize the cut, copy & paste strings in a resource bundle.
       String cut   = "Cut";    //rb.getString("popup.cut");    // I18N
       String copy  = "Copy";   //rb.getString("popup.copy");   // I18N
       String paste = "Paste";  //rb.getString("popup.paste");  // I18N

       String[] labels = new String[] {cut, copy, paste};
       String[] cmds   = new String[] {"cut", "copy", "paste"};

       for (int i = 0; i < labels.length; ++i)
       {
           menuItems[i] = new JMenuItem(labels[i]);
           menuItems[i].setActionCommand(cmds[i]);
           menuItems[i].addActionListener(this);
           add(menuItems[i]);
       }
    }

    /**
     * Checks to see if a <code>java.lang.SecurityManager</code> is installed.
     *
     * @return  boolean value indicating whether access to the system clipboard is granted.
     * @see     System#getSecurityManager
     * @since   1.0
     */
    protected boolean checkClipboardAccess()
    {
        SecurityManager security = System.getSecurityManager();
        if (security != null)
        {
            try
            {
                security.checkSystemClipboardAccess();
                return true;
            }
            catch(SecurityException se)
            {
                System.err.println(se.getCause());
                System.err.println(se.getMessage());
                se.printStackTrace();
                return false;
            }
        }
        else
        {
            return true;
        }
    }

    /**
     * Sets the <code>clipboard</code> member if security checks are passed.
     * A <code>java.awt.HeadLessException</code> can be thrown by the
     * <code>getSystemClipboard</code> method if program is running within an
     * environment that does not support a mouse, video display or keyboard.
     *
     * @since 1.0
     * @see   java.awt.HeadlessException
     */
    protected void getClipboard()
    {
        if(checkClipboardAccess())
        {
            try
            {
                clipboard = host.getToolkit().getSystemClipboard();
            }
            catch(HeadlessException he)
            {
                System.err.println(he.getCause());
                System.err.println(he.getMessage());
                he.printStackTrace();
            }
        }
    }

    /**
     * Abstraction based on the Template Pattern that handles events
     * for cut, copy and paste commands.
     *
     * @param e <code>java.awt.event.ActionEvent</code> for capturing
     *          action command strings.
     * @see     java.awt.event.ActionListener
     * @see     java.awt.event.ActionEvent
     * @see     com.woven_media.colorsafe.AbstractPopup
     * @since   1.0
     */
    public void actionPerformed(ActionEvent e)
    {
        String cmd = e.getActionCommand();
        if(cmd.equals("cut"))  cut();
        if(cmd.equals("copy")) copy();
        if(cmd.equals("paste"))paste();
    }

    /**
     * Transfers data from the system clipboard.
     *
     * @since 1.0
     */
    protected abstract void paste();

    /**
     * Copies selected text to the system clipboard.
     *
     * @since 1.0
     */
    protected abstract void copy();

    /**
     * Clears text and transfers data to the system clipboard.
     *
     * @since 1.0
     */
    protected abstract void cut();

} // End class
